@@ -1,15 +1,18 @@
 """ 数据库 """
 import asyncio
-import os
-from typing import TYPE_CHECKING, AsyncGenerator, Callable
+from pathlib import Path
+from typing import TYPE_CHECKING, AsyncGenerator, Callable, Dict, List
 
 from nonebot import get_driver
 from nonebot.log import logger
 from nonebot.utils import is_coroutine_callable, run_sync
+from sqlalchemy.engine import make_url
 from sqlalchemy.ext.asyncio import create_async_engine
 from sqlmodel import SQLModel
 from sqlmodel.ext.asyncio.session import AsyncSession
 
+from nonebot_plugin_datastore.utils import get_caller_plugin_name
+
 from .config import plugin_config
 
 if TYPE_CHECKING:
@@ -18,7 +21,7 @@ if TYPE_CHECKING:
 
 _engine = None
 
-_pre_db_init_funcs = []
+_pre_db_init_funcs: Dict[str, List] = {}
 _post_db_init_funcs = []
 
 
@@ -30,7 +33,10 @@ def get_engine() -> "AsyncEngine":
 
 def pre_db_init(func: Callable) -> Callable:
     """数据库初始化前执行的函数"""
-    _pre_db_init_funcs.append(func)
+    name = get_caller_plugin_name()
+    if name not in _pre_db_init_funcs:
+        _pre_db_init_funcs[name] = []
+    _pre_db_init_funcs[name].append(func)
     return func
 
 
@@ -45,9 +51,10 @@ async def init_db():
     from .script.utils import run_upgrade
 
     # 执行数据库初始化前执行的函数
+    pre_db_init_funcs = [i for funcs in _pre_db_init_funcs.values() for i in funcs]
     cors = [
         func() if is_coroutine_callable(func) else run_sync(func)()
-        for func in _pre_db_init_funcs
+        for func in pre_db_init_funcs
     ]
     if cors:
         try:
@@ -79,10 +86,25 @@ async def init_db():
 if plugin_config.datastore_enable_database:
     # 创建数据文件夹
     # 防止数据库创建失败
-    os.makedirs(plugin_config.datastore_data_dir, exist_ok=True)
+    url = make_url(plugin_config.datastore_database_url)
+    if (
+        url.drivername.startswith("sqlite")
+        and url.database is not None
+        and url.database not in [":memory:", ""]
+    ):
+        database_path = Path(url.database)
+        database_path.parent.mkdir(parents=True, exist_ok=True)
+        logger.debug(f"创建数据库文件夹: {database_path.parent}")
+    # 创建数据库引擎
+    engine_options = {}
+    engine_options.update(plugin_config.datastore_engine_options)
+    engine_options.setdefault("echo", plugin_config.datastore_database_echo)
+    engine_options.setdefault("echo_pool", plugin_config.datastore_database_echo)
+    logger.debug(f"数据库连接地址: {plugin_config.datastore_database_url}")
+    logger.debug(f"数据库引擎参数: {engine_options}")
     _engine = create_async_engine(
         plugin_config.datastore_database_url,
-        echo=plugin_config.datastore_database_echo,
+        **engine_options,
     )
 
     get_driver().on_startup(init_db)
